# Motivation

This package is largely motivated by the use of swizzling in computer graphics. It is common to write [shaders](https://en.wikipedia.org/wiki/Shader) which use built-in vector types to represent many kinds of data, not just position. Notably, 4-component vectors are particularly efficient to use in hardware and map well to 3D locations using a fourth [homogeneous coordinate](https://en.wikipedia.org/wiki/Homogeneous_coordinates), which faciliates the application of [projective geometry](https://en.wikipedia.org/wiki/Projective_geometry) techniques.

Because of the efficiency of built-in vectors and special intrinsics defined for them (that may or may not have optimized hardware implementations), it is common for shaders to abuse 2, 3 or 4-component vectors outside of spatial descriptions, where for example 3 or 4-component vectors represent colors (depending on whether an alpha channel is used). In fact, the way shader code integrates with the graphics pipeline in graphics APIs [*requires*](https://www.khronos.org/opengl/wiki/Fragment#Fragment_shader_outputs) colors to be 4-component vectors in [fragment shaders](https://www.khronos.org/opengl/wiki/Fragment_Shader). Another example might be to use a 2-component vector to store a strength and a radius parameter for a blur computation algorithm. It does not mean that this is necessarily more efficient than having a user-defined structure with explicit fields, and it is certainly less intuitive to abuse vectors here, but such abuse is widespread in graphics programming.

To facilitate the integration of graphics programming code and techniques, it is therefore useful to represent certain operations that are specific to vectors and probably not expected outside of computer graphics, swizzling being one such operation. At its core, swizzling is really just a way to reorder data, using vectors as a way to organize such data.

The main appeal to represent swizzling as a function (and provide a convenient syntax for it) is that swizzles are extremely cheap on the GPU. Therefore, a shader compiler such as [SPIRV.jl](https://github.com/serenity4/SPIRV.jl) may want to override [`swizzle`](@ref) methods to emit a single GPU intrinsic for the operation, hinging on the existence of such a function in the first place.

After [an attempt](https://github.com/JuliaArrays/StaticArrays.jl/pull/1221) to allow swizzling in `getproperty`/`setproperty!` in [StaticArrays](https://github.com/JuliaArrays/StaticArrays.jl) (containing the main implementation of small statically-sized vectors), it was decided that having it in an external package was preferable.
